#!/usr/local/bin/tclsh
#
# pfile_cat
#
# Read a pfile and write out one of its utterances to stdout in 
# online_feature_format
#
# 1997sep25 dpwe@icsi.berkeley.edu
# $Header: /n/yam/da/dpwe/projects/recogviz/RCS/pfile_cat.tcl,v 1.3 1998/02/22 22:02:11 dpwe Exp $

#if {[catch {set SPEECH_DIR $env(SPEECH_DIR)}]} {
#    set SPEECH_DIR /u/drspeech
#}
#if {[catch {set SPEECH_ARCH $env(SPEECH_ARCH)}]} {
#    if {[catch {set SPEECH_ARCH [exec $SPEECH_DIR/share/bin/speech_arch]}]} {
#	set SPEECH_ARCH sun4
#    }
#}
#set SPEECH_ARCHDIR "$SPEECH_DIR/$SPEECH_ARCH"
## For standard extensions e.g. APRL
#lappend auto_path "$SPEECH_ARCHDIR/lib"
## Stuff e.g. for breadi lib (libdpweutils_tcl.so)
#lappend auto_path "/u/dpwe/$SPEECH_ARCH/lib"
#lappend auto_path "/u/dpwe/share/lib/tcl"
## Get ICSI arg handling
#package require Dpwe_Utilfns

# Assume that most of path was set up by TCLLIBPATH on entry

set DPWE "/u/dpwe"
# Where to find the general Tcl utils
lappend auto_path "$DPWE/share/lib"
# Directory of our widgets (e.g. CanvTimeLine)
lappend auto_path "$DPWE/share/lib/itcl"

# Load the basic utils, including "SpeechArch"
package require Dpwe_Utilfns

# Stuff e.g. for breadi lib (libdpweutils_tcl.so)
lappend auto_path "$DPWE/[SpeechArch]/lib"

# Get the breadi extensions
package require Dpweutils_tcl

# Get the pfile access extensions
package require Pfif

proc Usage {argv0} {
    puts stderr "usage: $argv0 pfile=<file> [startutt=0] [nutts=1]"
    puts stderr "  will copy the features corresponding to <nutts>"
    puts stderr "  utterances from the pfile to stdout in online feature"
    puts stderr "  format, starting from utterance <startutt>."
    exit -1
}

set pfile ""
set startutt 0
set nutts 1
# Get the ICSI args
GetICSIargs "pfile startutt nutts"

if {$pfile == ""} {
    Usage $argv0
}

set out stdout
fconfigure $out -translation binary -buffering none

# OK, looking good.  Try to read the pfile
set pf [pfileif #auto]
$pf open $pfile "r"

set numftrs [$pf getNumFeatures]
if {$numftrs < 1} {
    puts stderr "$argv0: nftrs of $numftrs in $pfile is bogus"
    exit -1
}

for {set i 0} {$i < $nutts} {incr i} {
    set dataset [expr $startutt + $i]
    if {$dataset >= [$pf getNumUtts]} {
	puts stderr "Cannot get dataset $dataset in file with [$pf getNumUtts] datasets"
	exit -1
    }
    $pf seek $dataset 0
    set nrows [$pf numFramesForUtt $dataset]
    for {set j 0} {$j < $nrows} {incr j} {
	set data [$pf readFrame]
	# Write it out in online feature format
	set flag 0
	if {$j == [expr $nrows - 1]} {
	    # last frame
	    set flag 128
	}
	bwritei $out $flag 1
	for {set k 0} {$k < $numftrs} {incr k} {
	    bwritef $out [$data Val $k]
	}
    }
}

flush $out

exit 0


